# fs

[![JSR](https://jsr.io/badges/@miyauci/fs)](https://jsr.io/@miyauci/fs)
[![codecov](https://codecov.io/gh/TomokiMiyauci/fs/graph/badge.svg?token=KJNNOLNBOD)](https://codecov.io/gh/TomokiMiyauci/fs)
[![GitHub](https://img.shields.io/github/license/TomokiMiyauci/fs)](https://github.com/TomokiMiyauci/fs/blob/main/LICENSE)
[![semantic-release: angular](https://img.shields.io/badge/semantic--release-angular-e10079?logo=semantic-release)](https://github.com/semantic-release/semantic-release)
[![standard-readme compliant](https://img.shields.io/badge/readme%20style-standard-brightgreen.svg)](https://github.com/RichardLitt/standard-readme)

[File System Standard](https://whatpr.org/fs/165.html), based on WHATWG spec
reference implementation.

## Table of Contents <!-- omit in toc -->

- [Install](#install)
- [Usage](#usage)
  - [Supported Runtime](#supported-runtime)
  - [FileSystemObserver](#filesystemobserver)
    - [`FileSystemObserver` with `LocalFileSystem`](#filesystemobserver-with-localfilesystem)
- [Documents](#documents)
- [API](#api)
- [Contributing](#contributing)
- [License](#license)

## Install

deno:

```bash
deno add @miyauci/fs
```

node:

```bash
npx jsr add @miyauci/fs
```

## Usage

```ts
import { LocalFileSystem } from "@miyauci/fs/$RUNTIME";

const fs = new LocalFileSystem("path/to/dir"); // default is "."
const handle = await fs.getDirectory();
```

This allows operations to be performed on the local file system using the File
System API.

### Supported Runtime

The following run-times are supported:

| Runtime           | Import Specifier   |
| ----------------- | ------------------ |
| Deno              | `@miyauci/fs/deno` |
| Node.js, Deno[^1] | `@miyauci/fs/node` |

[^1]: Available on Deno Deploy. See
    [To use on Deno Deploy](./docs/faq.md#to-use-on-deno-deploy)

Each endpoint is only guaranteed to run at the corresponding runtime.

### FileSystemObserver

[FileSystemObserver](https://whatpr.org/fs/165.html#api-filesystemobserver)
monitors changes to the `FileSystemHandle`.

```ts
import {
  type FileSystemHandle,
  FileSystemObserver,
  type FileSystemObserverCallback,
} from "@miyauci/fs";

declare const handle: FileSystemHandle;
declare const callback: FileSystemObserverCallback;
const observer = new FileSystemObserver(callback);

await observer.observe(handle);
```

#### `FileSystemObserver` with `LocalFileSystem`

To use `FileSystemObserver` with `LocalFileSystem`, you must call
`LocalFileSystem#watch`.

```ts
import {
  FileSystemObserver,
  type FileSystemObserverCallback,
} from "@miyauci/fs";
import { LocalFileSystem } from "@miyauci/fs/$RUNTIME";

const fs = new LocalFileSystem();
const handle = await fs.getDirectory();
declare const callback: FileSystemObserverCallback;
const observer = new FileSystemObserver(callback);

fs.watch();
await observer.observe(handle, { recursive: true });

await handle.getFileHandle("file.txt", { create: true });
```

`LocalFileSystem#unwatch` will stop the monitoring.

## Documents

- [FAQ](./docs/faq.md)

## API

The API is published according to the following criteria:

- WebIDL
- Defined as an algorithm in the specification

See [jsr doc](https://jsr.io/@miyauci/fs) for all APIs.

## Contributing

See [CONTRIBUTING.md](CONTRIBUTING.md)

## License

[MIT](LICENSE) © 2024 Tomoki Miyauchi
